from .base import BaseDevice


class BaseMul(BaseDevice):

    def __init__(self, ip, device_type='relay', **kwargs):
        super().__init__(ip, device_type, **kwargs)
        self.status = list(['close'] * self.socket_count)

    @property
    def socket_count(self):
        return 0

    async def do(self, action, value=None):
        device_id = action[-1:0]
        if device_id.isnumeric() and 0 < int(device_id) <= self.socket_count:
            device_id = int(device_id - 1)
        else:
            device_id = False

        if action == 'get_count':
            return self.socket_count
        elif action == 'get_status_all':
            return 'on' if all(status == 'open' for status in self.status) else 'off'
        elif action[:-1] == 'get_status' and device_id:
            return 'on' if self.status[device_id] == 'open' else 'off'
        elif action == 'turn_on_socket' and device_id:
            await self.turn_on(device_id)
        elif action == 'turn_off_socket' and device_id:
            await self.turn_off(device_id)
        elif action == 'turn_on_all':
            await self.turn_on_all()
        elif action == 'turn_off_all':
            await self.turn_off_all()
        else:
            return await super().do(action, value)

    """
        获取状态
        req: lan_phone%28-d9-8a-XX-XX-XX%XXXXXXXX%check%relay
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%open1,close2,open3,close4%rack
    """

    async def update(self, **kwargs):
        res = await self.send_message('check', **kwargs)
        for index, t in enumerate(res.split(',')):
            self.status[index] = t[:-1]

    """
        开启插孔
        req: lan_phone%28-d9-8a-XX-XX-XX%XXXXXXXX%openX%relay
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%open%rack
    """

    async def turn_on(self, index, **kwargs):
        if self.status[index] != 'open':
            await self.send_message('open' + str(index + 1), **kwargs)
            self.status[index] = 'open'

    """
        关闭插孔
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%closeX%relay
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%close%rack
    """

    async def turn_off(self, index, **kwargs):
        if self.status[index] != 'close':
            await self.send_message('close' + str(index + 1), **kwargs)
            self.status[index] = 'close'

    """
        全部开启
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%openall%relay
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%close%rack
    """

    async def turn_on_all(self, **kwargs):
        if any(status != 'open' for status in self.status):
            await self.send_message('openall', **kwargs)
            self.status = list('open' for _ in self.status)

    """
        全部关闭
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%closeall%relay
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%close%rack
    """

    async def turn_off_all(self, **kwargs):
        if any(status != 'close' for status in self.status):
            await self.send_message('closeall', **kwargs)
            self.status = list('close' for _ in self.status)



import time
import asyncio
from .. import error

DEFAULT_GROUP = 'pykonkeio'


class IRMixin:
    def __init__(self):
        self.ir_learning = False

    @property
    def is_support_ir(self):
        raise NotImplementedError()

    @staticmethod
    def is_ir_action(action):
        return action in ['learn_ir', 'emit_ir', 'remove_ir']

    async def ir_do(self, action, value=None):
        if not value:
            raise error.IllegalValue
        if action == 'learn_ir':
            result = await self.ir_learn(value)
            return 'success' if result else 'failed'
        elif action == 'emit_ir':
            await self.ir_emit(value)
        elif action == 'remove_ir':
            await self.ir_remove(value)

    """
        开始学习红外信号
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3031#learn#xxxx#xxx%uart
        res: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3031#learn#xxxx#xxx%uack

        检查学习进度
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%check#3031#learn#xxxx#xxx%uart
        res(进行): lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%check#3031#learn#xxxx#xxx%uack
        res(成功): lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%check#3031#learn#xxxx#xxx#ok%uack
        res(失败): lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%check#3031#learn#xxxx#xxx#failed%uack
    """
    async def ir_learn(self, ir_id, group=DEFAULT_GROUP, timeout=30):
        if self.is_support_ir:
            self.ir_learning = True
            cmd = 'operate#3031#learn#%s#%s' % (group, ir_id)
            cmd1 = 'check#3031#learn#%s#%s' % (group, ir_id)
            await self.send_message(cmd, 'uart')

            start = time.time()
            while self.ir_learning:
                if time.time() - start >= timeout:
                    return await self.ir_quit()
                await asyncio.sleep(1)
                res = await self.send_message(cmd1, 'uart')
                if res != cmd1:
                    return res.endswith('#ok')

    """
        停止学习红外信号
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3031#quit%uart
        res: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3031#quit%uack
    """
    async def ir_quit(self):
        if self.is_support_ir and self.ir_learning:
            cmd = 'operate#3031#quit'
            self.ir_learning = False
            await self.send_message(cmd, 'uart')

    """
        发射红外信号
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3031#emit#xxxx#xxx%uart
        res: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3031#emit#xxxx#xxx%uack
    """
    async def ir_emit(self, ir_id, group=DEFAULT_GROUP):
        if self.is_support_ir:
            cmd = 'operate#3031#emit#%s#%s' % (group, ir_id)
            await self.send_message(cmd, 'uart')

    """
        删除红外信号
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3031#deletekey#xxxx#xxx%uart
        res: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3031#deletekey#xxxx#xxx%uack
    """
    async def ir_remove(self, ir_id, group=DEFAULT_GROUP):
        if self.is_support_ir:
            cmd = 'operate#3031#deletekey#%s#%s' % (ir_id, group)
            await self.send_message(cmd, 'uart')

    """
        删除红外信号组
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3031#delete#xxxx%uart
        res: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3031#delete#xxxx%uack
    """
    async def ir_remove_group(self, group=DEFAULT_GROUP):
        if self.is_support_ir:
            cmd = 'operate#3031#delete#%s' % group
            await self.send_message(cmd, 'uart')



import argparse
import logging
import asyncio

from . import socket
from . import manager
from . import error

help_content = '''
usage: konkeio [action] [device] [address] [value] [--verbose]

Supported devices and actions supported by each device:
global: search help
k2:     get_status turn_[on/off] get_usb_status turn_[on/off]_usb get_light_status turn_[on/off]_light get_power
minik:  get_status turn_[on/off] learn_ir emit_ir remove_ir
micmul: get_count get_status_all get_status[1/2/3/4] turn_[on/off]_all turn_[on/off]_socket[1/2/3/4]
mul:    get_count get_status_all get_status[1/2/3] get_usb_count get_usb_status_all get_usb_status[1/2] 
        turn_[on/off]_all turn_[on/off]_socket[1/2/3] turn_[on/off]_usb[1/2] 
klight: get_status get_brightness get_color turn_[on/off] set_brightness set_color
kbulb:  get_status get_brightness get_ct turn_[on/off] set_brightness set_ct

* each action starts with 'set_' must provide a value parameter

Value Format:
color:      r,g,b
ct:         2700-6500
brightness: 0-100

Example:
konkeio search
konkeio turn_on minik 192.168.0.64
konkeio get_status minik 192.168.0.64
konkeio turn_on_usb k2 192.168.0.64
konkeio turn_off_light k2 192.168.0.64
konkeio get_count micmul 192.168.0.64
konkeio turn_on_socket3 micmul 192.168.0.64
konkeio get_status2 mul 192.168.0.64
konkeio turn_off_all mul 192.168.0.64
konkeio get_brightness klight 192.168.0.64
konkeio set_color klight 192.168.0.64 255,255,0
konkeio set_ct bulb 192.168.0.64 3400
konkeio turn_off bulb 192.168.0.64
'''


def print_device(ip, mac, password, *_):
    print('ip: %s\nmac: %s\npassword: %s\n\n' % (ip, mac, password))


async def main():
    parser = argparse.ArgumentParser(prog="konkeio", epilog=help_content,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('action', help='action name')
    parser.add_argument('device', help='device type', nargs='?')
    parser.add_argument('address', help='device ip address', nargs='?')
    parser.add_argument('value', help='action value', nargs='?')
    parser.add_argument('-v', dest='verbose', help='show debug', default=False, action="store_true")

    args = parser.parse_args()

    if args.verbose:
        logging.root.setLevel(logging.DEBUG)

    if args.action == 'search':
        try:
            ip = args.device or '255.255.255.255'
            await manager.search(ip=ip, callback=print_device)
        except error.Timeout:
            pass
        except error.KonkeError as err:
            print(err)
    elif args.action == 'help':
        print(help_content)
    elif args.action == 'send':
        ip, mac, password, *_ = await manager.get_device_info(args.device)
        print(await socket.send_message((ip, mac, password, args.address, args.value)))
    else:
        try:
            if args.address is None or args.device is None:
                print('Usage: konkeio [action] [device] [address] [value] [--verbose]')
                print("Try 'konkeio help' for more information.")
                exit(0)

            device = manager.get_device(args.address, args.device)
            await device.update()
            res = await device.do(args.action, args.value)

            print(res if res else 'success')

        except error.KonkeError as err:
            print(err)

    current = asyncio.Task.current_task()
    for task in asyncio.Task.all_tasks():
        if task != current:
            task.cancel()


if __name__ == "__main__":
    logging.basicConfig(level=logging.WARN,
                        datefmt='%Y/%m/%d %H:%M:%S',
                        format='%(asctime)s %(levelname)s %(message)s')

    loop = asyncio.get_event_loop()
    try:
        loop.run_until_complete(main())
    except KeyboardInterrupt:
        pass
    loop.stop()




from .basetoggle import BaseToggle


class K1(BaseToggle):
    async def send_message(self, action, action_type=None, retry=2, loop=None):
        res = await super().send_message(action, 'request', retry=retry, loop=loop)
        if res[0:7] == 'confirm':
            data = await super().send_message(res, 'request', retry=retry, loop=loop)
            return data[3]
        else:
            return res



from .basemul import BaseMul

USB_COUNT = 2
SOCKET_COUNT = 3


class Mul(BaseMul):

    def __init__(self, ip, **kwargs):
        super().__init__(ip, 'relay', **kwargs)
        self.usb_count = USB_COUNT
        self.usb_status = ['close'] * USB_COUNT

    @property
    def socket_count(self):
        return SOCKET_COUNT

    async def do(self, action, value=None):
        usb_device_id = action[-1:0]
        if usb_device_id.isnumeric() and 0 < int(usb_device_id) <= self.socket_count:
            usb_device_id = int(usb_device_id - 1)
        else:
            usb_device_id = False

        if action == 'get_usb_count':
            return USB_COUNT
        elif action == 'get_usb_status_all':
            return 'on' if all(status == 'open' for status in self.usb_status) else 'off'
        elif action[:-1] == 'get_usb_status' and usb_device_id:
            return 'on' if self.usb_status[usb_device_id] == 'open' else 'off'
        elif action == 'turn_on_usb' and usb_device_id:
            await self.turn_on_usb(usb_device_id)
        elif action == 'turn_off_usb' and usb_device_id:
            await self.turn_off_usb(usb_device_id)
        else:
            return await super().do(action, value)

    """
        获取状态
        req: lan_phone%28-d9-8a-XX-XX-XX%XXXXXXXX%check%usb
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%open1,close2%uack
    """
    async def update(self, **kwargs):
        await super().update(**kwargs)

        for index, t in enumerate((await self.send_message('check', 'usb', **kwargs)).split(',')):
            self.usb_status[index] = t[:-1]

    """
        打开USB
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%openX%usb
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%open%uack
    """
    async def turn_on_usb(self, index, **kwargs):
        if self.usb_status[index] != 'open':
            await self.send_message('open' + str(index + 1), 'usb', **kwargs)
            self.usb_status[index] = 'open'

    """
        关闭USB
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%open%usb
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%close%uack
    """
    async def turn_off_usb(self, index, **kwargs):
        if self.usb_status[index] != 'close':
            await self.send_message('close' + str(index + 1), 'usb', **kwargs)
            self.usb_status[index] = 'close'



from .basetoggle import BaseToggle
from .. import utils
from .. import error


class KLight(BaseToggle):

    def __init__(self, ip, **kwargs):
        super().__init__(ip, 'klight', **kwargs)
        self.color = [0, 0, 0]
        self.brightness = 0
        self.m = 0

    async def do(self, action, value=None):
        if action == 'get_brightness':
            return self.brightness
        elif action == 'get_color':
            return self.color
        elif action == 'set_brightness':
            await self.set_brightness(value)
        elif action == 'set_color':
            await self.set_color(*value.split(','))
        else:
            return await super().do(action, value)

    """
        获取状态
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%check%klight
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%open#x#x#x#x#1,x#1&#x#x#x#x#2,x#1&#x#x#x#x#3,x#1&#x#x#x#x#5,x#1
             %klack
    """
    async def update(self, **kwargs):
        if not self.is_online:
            await super().update(**kwargs)
        try:
            m1, m2, *_ = (await self.send_message('check', **kwargs)).split('&')

            self.status, *_ = m1.split('#')

            _, r, g, b, w, t, _ = m2.split('#')
            self.color = [int(r), int(g), int(b)]
            self.brightness = int(w)
            m, _ = t.split(',')
            self.m = int(m)
        except ValueError:
            raise error.ErrorMessageFormat

    """
        调整亮度
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%set#r#g#b#w#2%klight
    """
    async def set_brightness(self, w, **kwargs):
        try:
            utils.check_number(w, 0, 100)
        except ValueError:
            raise error.IllegalValue('brightness should between 0 and 100')

        if self.brightness == int(w):
            return

        [r, g, b] = self.color
        await self.send_message('set#%s#%s#%s#%s#1,%s#1' % (r, g, b, w, self.m), **kwargs)
        self.brightness = int(w)

        if self.brightness == 0:
            await self.turn_off()

    """
        调整颜色
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%open%klight
    """
    async def set_color(self, r=None, g=None, b=None, **kwargs):
        try:
            utils.check_number(r, 0, 255)
            utils.check_number(g, 0, 255)
            utils.check_number(b, 0, 255)
        except ValueError:
            raise error.IllegalValue('illegal color value')

        if self.color != [int(r), int(g), int(b)]:
            await self.send_message('set#%s#%s#%s#%s#1,%s#1' % (r, g, b, self.brightness, self.m), **kwargs)
            self.color = [int(r), int(g), int(b)]

    async def turn_on(self, **kwargs):
        await super().turn_on(**kwargs)
        if self.brightness == 0:
            await self.set_brightness(50)

        # @todo rgb 0,0,0



from .basetoggle import BaseToggle
from ..mixin.ir import IRMixin


class MiniK(BaseToggle, IRMixin):
    def __init__(self, ip, **kwargs):
        super().__init__(ip, **kwargs)
        self.is_pro = None

    @property
    def is_support_ir(self):
        return self.is_pro

    async def do(self, action, value=None):
        if self.is_support_ir and self.is_ir_action(action):
            return await self.ir_do(action, value)
        else:
            return await super().do(action, value)

    async def fetch_info(self):
        status = await super().fetch_info()
        if status:
            self.is_pro = status.find('#hv2.') > 0



import time
import logging
import asyncio

from . import error
from . import socket

_LOGGER = logging.getLogger(__name__)
_devices = {}
_device_info = {}


async def search(ip='255.255.255.255', callback=None, loop=None):
    datetime = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
    socket.send(ip, 'mac', 'nopassword', datetime, 'heart')

    future = asyncio.Future(loop=loop)

    def message_handler(*data):
        if data[4] != 'hack':
            return
        if callback is not None:
            callback(*data)
        if ip == data[0] and not future.done():
            future.set_result(data)
            _device_info[data[0]] = data

    socket.add_message_handler(message_handler)
    try:
        return await asyncio.wait_for(future, timeout=2)
    except asyncio.TimeoutError:
        raise error.Timeout
    finally:
        socket.remove_message_handler(message_handler)


def get_device(ip, device_type=None):
    if ip in _devices:
        return _devices[ip]

    if device_type is None:
        from .device.basetoggle import BaseToggle
        device = BaseToggle(ip)
    elif device_type == 'k1':
        from .device.k1 import K1
        device = K1(ip)
    elif device_type == 'k2':
        from .device.k2 import K2
        device = K2(ip)
    elif device_type == 'minik':
        from .device.minik import MiniK
        device = MiniK(ip)
    elif device_type == 'micmul':
        from .device.micmul import MicMul
        device = MicMul(ip)
    elif device_type == 'mul':
        from .device.mul import Mul
        device = Mul(ip)
    elif device_type == 'klight':
        from .device.klight import KLight
        device = KLight(ip)
    elif device_type == 'kbulb':
        from .device.kbulb import KBulb
        device = KBulb(ip)
    else:
        raise error.DeviceNotSupport('device %s not support' % device_type)

    _devices[ip] = device
    return device


def clear_device_info(ip=None):
    global _device_info
    if ip:
        del _device_info[ip]
    else:
        _device_info = {}


async def get_device_info(ip):
    if ip not in _device_info:
        await search(ip)
    return _device_info[ip]



import time
import asyncio
from .. import error
from abc import abstractmethod, ABCMeta

DEFAULT_GROUP = 'pykonkeio'


class RFMixin(metaclass=ABCMeta):
    def __init__(self):
        self.rf_learning = False

    @property
    @abstractmethod
    def is_support_rf(self):
        return False

    @staticmethod
    def is_rf_action(action):
        return action in ['learn_rf', 'emit_rf', 'remove_rf']

    async def rf_do(self, action, value=None):
        if not value:
            raise error.IllegalValue
        if action == 'learn_rf':
            result = await self.rf_learn(value)
            return 'success' if result else 'failed'
        elif action == 'emit_rf':
            await self.rf_emit(value)
        elif action == 'remove_rf':
            await self.rf_remove(value)
        
    """
        开始学习射频信号
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3035#learn#xxxx#xxx%uart
        res: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3035#learn#xxxx#xxx%uack

        检查学习进度
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%check#3035#learn#xxxx#xxx%uart
        res(进行): lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%check#3035#learn#xxxx#xxx%uack
        res(成功): lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%check#3035#learn#xxxx#xxx#ok%uack
        res(失败): lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%check#3035#learn#xxxx#xxx#failed%uack
    """
    async def rf_learn(self, rf_id, group=DEFAULT_GROUP, timeout=30):
        if self.is_support_rf:
            self.rf_learning = True
            cmd = 'operate#3035#learn#%s#%s' % (group, rf_id)
            cmd1 = 'check#3035#learn#%s#%s' % (group, rf_id)
            await self.send_message(cmd, 'uart')

            start = time.time()
            while self.rf_learning:
                if time.time() - start >= timeout:
                    return await self.rf_quit()
                await asyncio.sleep(1)
                res = await self.send_message(cmd1, 'uart')
                if res != cmd1:
                    return res.endswith('#ok')

    """
        停止学习射频信号
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3035#quit%uart
        res: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3035#quit%uack
    """
    async def rf_quit(self):
        if self.is_support_rf and self.rf_learning:
            cmd = 'operate#3035#quit'
            self.rf_learning = False
            await self.send_message(cmd, 'uart')

    """
        发射射频信号
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3035#emit#xxxx#xxx%uart
        res: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3035#emit#xxxx#xxx%uack
    """
    async def rf_emit(self, rf_id, group=DEFAULT_GROUP):
        if self.is_support_rf:
            cmd = 'operate#3035#emit#%s#%s' % (group, rf_id)
            await self.send_message(cmd, 'uart')

    """
        删除射频信号
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3035#deletekey#xxxx#xxx%uart
        res: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3035#deletekey#xxxx#xxx%uack
    """
    async def rf_remove(self, rf_id, group=DEFAULT_GROUP):
        if self.is_support_rf:
            cmd = 'operate#3035#delete#%s#%s' % (group, rf_id)
            await self.send_message(cmd, 'uart')

    """
        删除射频信号组
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3035#delete#xxxx%uart
        res: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%operate#3035#delete#xxxx%uack
    """
    async def rf_remove_group(self, group=DEFAULT_GROUP):
        if self.is_support_rf:
            cmd = 'operate#3035#delete#%s' % group
            await self.send_message(cmd, 'uart')



import socket
import math
import logging
import asyncio
from . import error
from . import utils

_PORT = 27431

_LOGGER = logging.getLogger(__name__)

_requests = []
_receivers_count = 0
_message_handlers = []
_receive_task = None

_sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
_sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
_sock.setblocking(False)


def send(ip, mac, password, action, action_type, device_type='lan_phone'):
    address = (ip, _PORT)
    cmd = '%s%%%s%%%s%%%s%%%s' % (device_type, mac, password, action, action_type)
    message = utils.encrypt(cmd)
    _sock.sendto(message, address)
    _LOGGER.debug('send %s %s', ip, cmd)


async def _do_receive(loop=None):
    def message_handler(*_):
        message, (address, _) = _sock.recvfrom(256)
        message = utils.decrypt(message)
        _LOGGER.debug('receive %s %s', address, message or '(empty)')

        device, *data = message.split('%')
        if len(data) < 4 or device != 'lan_device':
            return _LOGGER.error('incorrect response %s', message)

        mac, _, _, response_type = data

        for request in _requests:
            if not request['future'].done() and request['mac'] == mac and request['request_type'].startswith(response_type[:-3]):
                request['future'].set_result((address, *data))
                break

        for callback in _message_handlers:
            callback(address, *data)

    loop = loop or asyncio.get_event_loop()
    loop.add_reader(_sock.fileno(), message_handler, None)

    try:
        await asyncio.sleep(math.inf)
    except asyncio.CancelledError:
        loop.remove_reader(_sock.fileno())


async def send_message(params, retry=2, loop=None, **kwargs):
    request = {
        'mac': params[1],
        'request_type': params[4],
        'future': asyncio.Future(loop=loop)
    }
    _requests.append(request)

    send(*params)
    add_receiver(loop=loop)

    try:
        return await asyncio.wait_for(request['future'], timeout=1)
    except asyncio.TimeoutError:
        if retry > 0:
            return await send_message(params, retry=retry-1, loop=loop)
        raise error.Timeout('connect timeout')
    finally:
        _requests.remove(request)
        remove_receiver()


def add_message_handler(handler, loop=None):
    _message_handlers.append(handler)
    add_receiver(loop=loop)


def remove_message_handler(handler):
    _message_handlers.remove(handler)
    remove_receiver()


def add_receiver(loop=None):
    global _receivers_count, _receive_task
    if _receivers_count == 0:
        _receive_task = asyncio.ensure_future(_do_receive(loop=loop))
    _receivers_count += 1


def remove_receiver():
    global _receivers_count
    _receivers_count -= 1
    if _receivers_count == 0:
        _receive_task and _receive_task.cancel()



class KonkeError(Exception):
    def __init__(self, message=''):
        self.message = message

    def __str__(self):
        return 'Error: ' + (self.message or self.__class__.__name__)


class Timeout(KonkeError):
    def __init__(self, message='request time out'):
        super().__init__(message)


class DeviceOffline(KonkeError):
    def __init__(self, message='device is offline'):
        super().__init__(message)


class ErrorMessageFormat(KonkeError):
    def __init__(self, message='wrong message'):
        super().__init__(message)


class DeviceNotSupport(KonkeError):
    def __init__(self, message='device not support'):
        super().__init__(message)


class IllegalAction(KonkeError):
    def __init__(self, message='action not support'):
        super().__init__(message)


class IllegalValue(KonkeError):
    def __init__(self, message='illegal value'):
        super().__init__(message)



from .basetoggle import BaseToggle
from ..mixin.ir import IRMixin
from ..mixin.rf import RFMixin


class K2(BaseToggle, IRMixin, RFMixin):

    def __init__(self, ip, **kwargs):
        super().__init__(ip, 'relay', **kwargs)
        self.light_status = 'close'
        self.usb_status = 'close'
        self.rf_module = False
        self.ir_module = False

    @property
    def is_support_rf(self):
        return self.rf_module

    @property
    def is_support_ir(self):
        return self.ir_module

    async def fetch_info(self):
        status = await super().fetch_info()
        if status:
            self.ir_module = status.find('#ir_module#on') > 0
            self.rf_module = status.find('#rf_module#on') > 0

    async def do(self, action, value=None):
        if action == 'get_usb_status':
            return 'on' if self.usb_status == 'open' else 'off'
        elif action == 'get_light_status':
            return 'on' if self.light_status == 'open' else 'off'
        elif action == 'get_power':
            return await self.get_power()
        elif action == 'turn_on_usb':
            await self.turn_on_usb()
        elif action == 'turn_off_usb':
            await self.turn_off_usb()
        elif action == 'turn_on_light':
            await self.turn_on_light()
        elif action == 'turn_off_light':
            await self.turn_off_light()
        elif self.is_support_ir and self.is_ir_action(action):
            return await self.ir_do(action, value)
        elif self.is_support_rf and self.is_rf_action(action):
            await self.rf_do(action, value)
        else:
            return await super().do(action, value)

    async def update(self, **kwargs):
        update_type = kwargs['type'] if hasattr(kwargs, 'type') else None

        if update_type is None or update_type == 'relay':
            await super().update(**kwargs)

        if update_type is None or update_type == 'usb':
            self.usb_status = await self.send_message('check', 'usb', **kwargs)

        if update_type is None or update_type == 'light':
            self.light_status = await self.send_message('check', 'light', **kwargs)

    """
        打开USB
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%open%usb
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%open%uack
    """
    async def turn_on_usb(self, **kwargs):
        if self.usb_status != 'open':
            await self.send_message('open', 'usb', **kwargs)
            self.usb_status = 'open'
            await self.fetch_info()

    """
        关闭USB
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%open%usb
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%close%uack
    """
    async def turn_off_usb(self, **kwargs):
        if self.usb_status != 'close':
            await self.send_message('close', 'usb', **kwargs)
            self.usb_status = 'close'
            self.rf_module = False
            self.ir_module = False

    """
        打开夜灯
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%open%light
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%open%lack
    """
    async def turn_on_light(self, **kwargs):
        if self.light_status != 'open':
            await self.send_message('open', 'light', **kwargs)
            self.light_status = 'open'

    """
        关闭夜灯
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%open%light
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%close%lack
    """
    async def turn_off_light(self, **kwargs):
        if self.light_status != 'close':
            await self.send_message('close', 'light', **kwargs)
            self.light_status = 'close'

    """
        获取实时功率
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%check%power
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%close%pack
    """
    async def get_power(self, **kwargs):
        res = await self.send_message('check', 'power', **kwargs)

        _, _, power = res.split('#')
        return power



from .basetoggle import BaseToggle
from .. import utils
from .. import error


class KBulb(BaseToggle):

    def __init__(self, ip, **kwargs):
        super().__init__(ip, 'kbulb', **kwargs)
        self.brightness = 0
        self.ct = 0
        self.mode = 0

    async def do(self, action, value=None):
        if action == 'get_brightness':
            return self.brightness
        elif action == 'get_color_temperature' or action == 'get_ct':
            return self.ct
        elif action == 'get_mode':
            return self.mode
        elif action == 'set_brightness':
            await self.set_brightness(value)
        elif action == 'set_color_temperature' or action == 'set_ct':
            await self.set_ct(value)
        else:
            return await super().do(action, value)
    """
        获取状态
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%check%kbulb
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%open#2700,100,0&5161,50,6%kback
    """
    async def update(self, **kwargs):
        try:
            info = (await self.send_message('check')).split('&')

            [self.status, modes] = info[0].split('#')
            [mode1, *_] = modes.split('&')
            [ct, lum, mode] = mode1.split(',')

            self.ct = int(ct)
            self.brightness = int(lum)
            self.mode = int(mode)
        except ValueError:
            raise error.ErrorMessageFormat

    """
        调整亮度
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%%set#lum#xxx%kbulb
        res: lan_device%28-d9-8a-xx-xx-xx%XXXXXXXX%%set#lum#xxx%kback
    """
    async def set_brightness(self, w):
        try:
            utils.check_number(w, 0, 100)
        except ValueError:
            raise error.IllegalValue('brightness should between 0 and 100')

        if self.brightness == int(w):
            return

        await self.set_mode(1)
        await self.send_message('set#lum#%s' % w)
        self.brightness = int(w)
        self.status = 'open'

        if self.brightness == 0:
            await self.turn_off()

    """
        调整色温
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%%set#ctp#xxx%kbulb
        res: lan_device%28-d9-8a-xx-xx-xx%XXXXXXXX%%set#ctp#xxx%kback
    """
    async def set_ct(self, ct):
        try:
            utils.check_number(ct, 2700, 6500)
        except ValueError:
            raise error.IllegalValue('illegal color temperature')

        if self.ct != int(ct):
            await self.set_mode(1)
            await self.send_message('set#ctp#%s' % ct)
            self.ct = int(ct)
            self.status = 'open'

            if ct == 0:
                await self.turn_off()

    """
        调整模式
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%%set#mode#x%kbulb
        res: lan_device%28-d9-8a-xx-xx-xx%XXXXXXXX%%set#mode#x%kback
    """
    async def set_mode(self, mode):
        if self.mode != int(mode):
            await self.send_message('set#mode#%s' % mode)
            self.mode = int(mode)

    async def turn_on(self, **kwargs):
        await super().turn_on(**kwargs)
        if self.brightness == 0:
            await self.set_brightness(50)



from .base import BaseDevice


class BaseToggle(BaseDevice):

    def __init__(self, ip, device_type='relay', **kwargs):
        super().__init__(ip, device_type, **kwargs)
        self.status = None

    async def do(self, action, value=None):
        if action == 'get_status':
            return 'on' if self.status == 'open' else 'off'
        elif action == 'turn_on':
            await self.turn_on()
        elif action == 'turn_off':
            await self.turn_off()
        else:
            await super().do(action, value)

    """
        获取状态
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%check%relay
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%check%rack
    """
    async def update(self, **kwargs):
        self.status = await self.send_message('check', **kwargs)

    """
        打开开关
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%open%relay
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%open%rack
    """
    async def turn_on(self, **kwargs):
        if self.status != 'open':
            await self.send_message('open', **kwargs)
            self.status = 'open'

    """
        关闭开关
        req: lan_phone%28-d9-8a-xx-xx-xx%XXXXXXXX%open%relay
        res: lan_device%28-d9-8a-xx-xx-xx%nopassword%close%rack
    """
    async def turn_off(self, **kwargs):
        if self.status != 'close':
            await self.send_message('close', **kwargs)
            self.status = 'close'



from .. import manager
from .. import socket
from .. import error


class BaseDevice(object):

    def __init__(self, ip, device_type, loop=None):
        self.ip = ip
        self.device_type = device_type
        self.mac = None
        self.password = None
        self.is_online = False
        self.loop = loop

    async def fetch_info(self):
        try:
            _, self.mac, self.password, status, _ = await manager.get_device_info(self.ip)
            self.is_online = True
            return status
        except error.Timeout:
            self.is_online = False

    async def send_message(self, action, action_type=None, **kwargs):
        if not self.is_online:
            await self.fetch_info()

        if not self.is_online:
            raise error.DeviceOffline('device is offline')

        params = (self.ip, self.mac, self.password, action, action_type or self.device_type)

        try:
            data = await socket.send_message(params, **kwargs)
            return data[3]
        except error.Timeout:
            self.is_online = False
            raise error.DeviceOffline('device is offline')

    async def do(self, action, value=None):
        raise error.IllegalAction()



#!/usr/bin/env python3

from setuptools import setup, find_packages

with open("README.md", 'r', encoding='utf-8') as f:
    readme = f.read()

setup(
    name='pykonkeio',
    version='2.1.5',
    keywords=['konke', 'iot'],
    description='Python library for interfacing with konke smart appliances',
    long_description=readme,
    long_description_content_type="text/markdown",
    author='jedmeng',
    author_email='jedm@jedm.cn',
    url='https://github.com/jedmeng/python-konkeio',
    license='GPLv3',
    install_requires=[
        'pycryptodome>=3.6.0'
    ],
    packages=find_packages(include=['pykonkeio', 'pykonkeio.*']),
    include_package_data=True,
    python_requires='>=3.5',
    test_suite='tests',
    entry_points={
        'console_scripts': [
            'konkeio = pykonkeio.__main__:main',
        ]
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: OS Independent",
    ],
)
